@@ -25,27 +25,20 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	$Id: dummy.h,v 1.5 1999/07/15 21:11:37 wichert Exp $
+ *	$Id: dummy.h,v 1.8 2000/05/01 01:54:00 wichert Exp $
  */
 
 /* still unfinished */
-#define	sys_reboot		printargs
 #define	sys_ioperm		printargs
 #define	sys_syslog		printargs
 #define	sys_iopl		printargs
 #define	sys_vm86old		printargs
 #define	sys_modify_ldt		printargs
 #define	sys_get_kernel_syms	printargs
-#define	sys_quotactl		printargs
 #define	sys_bdflush		printargs
 #define	sys_sysfs		printargs
 #define	sys_afs_syscall		printargs
 
-#define sys_fdatasync		printargs
-#define sys_mlock		printargs
-#define sys_munlock		printargs
-#define sys_mlockall		printargs
-#define sys_munlockall		printargs
 #define sys_sched_setparam	printargs
 #define sys_sched_getparam	printargs
 #define sys_sched_setscheduler	printargs
@@ -60,12 +53,15 @@
 #define	sys_umount		sys_chdir
 #define	sys_swapon		sys_chdir
 #define	sys_swapoff		sys_chdir
-#define	sys_clone		sys_fork
 #define	sys_delete_module	sys_chdir
 #define	sys_fchdir		sys_close
 #define	sys_setfsuid		sys_setuid
 #define	sys_setfsgid		sys_setgid
 #define sys_nanosleep		sys_adjtime
+#define	sys_acct		sys_chdir
+#define sys_fdatasync		sys_close
+#define sys_mlock		sys_munmap
+#define sys_munlock		sys_munmap
 
 /* printargs does the right thing */
 #define	sys_setup		printargs
@@ -82,6 +78,7 @@
 #define	sys_vhangup		printargs
 #define	sys_idle		printargs
 #define	sys_getpgid		printargs
+#define sys_munlockall		printargs
 
 /* subcall entry points */
 #define	sys_socketcall		printargs
@@ -92,7 +89,6 @@
 #define	sys_gtty		printargs
 #define	sys_ftime		printargs
 #define	sys_prof		printargs
-#define	sys_acct		printargs
 #define	sys_phys		printargs
 #define	sys_lock		printargs
 #define	sys_mpx			printargs
@@ -206,4 +202,7 @@
 #define sys_poll		printargs
 #define	sys_create_module	printargs
 #define	sys_init_module		printargs
+#define	sys_quotactl		printargs
+#define sys_mlockall		printargs
+#define	sys_reboot		printargs
 #endif
