@@ -61,7 +61,7 @@
 	{ 1,	0,	sys_umask,	"umask" },		/* 60 */
 	{ 1,	TF,	sys_chroot,	"chroot" },		/* 61 */
 	{ 2,	0,	sys_fstat,	"fstat" },		/* 62 */
-	{ 0,	0,	printargs,	"SYS_63" },		/* 63 */
+	{ 2,	TF,	sys_fstat64,	"fstat64" },		/* 63 */
 	{ 1,	0,	sys_getpagesize,"getpagesize" },	/* 64 */
 	{ 3,	0,	sys_msync,	"msync" },		/* 65 */
 	{ 0,	TP,	sys_vfork,	"vfork" },		/* 66 */
@@ -73,10 +73,10 @@
 	{ 1,	0,	printargs,	"vadvise" },		/* 72 */
 	{ 2,	0,	sys_munmap,	"munmap" },		/* 73 */
 	{ 3,	0,	sys_mprotect,	"mprotect" },		/* 74 */
-	{ 3,	0,	printargs,	"madvise" },		/* 75 */
+	{ 3,	0,	sys_madvise,	"madvise" },		/* 75 */
 	{ 1,	0,	sys_vhangup,	"vhangup" },		/* 76 */
-	{ 2,	0,	printargs,	"SYS_77" },		/* 77 */
-	{ 3,   0,	sys_mincore,	"mincore" },		/* 78 */
+	{ 2,	TF,	printargs,	"truncate64" },		/* 77 */
+	{ 3,	0,	sys_mincore,	"mincore" },		/* 78 */
 	{ 2,	0,	sys_getgroups,	"getgroups" },		/* 79 */
 	{ 2,	0,	sys_setgroups,	"setgroups" },		/* 80 */
 	{ 1,	0,	sys_getpgrp,	"getpgrp" },		/* 81 */
@@ -130,21 +130,22 @@
 	{ 2,	TF,	sys_truncate,	"truncate" },		/* 129 */
 	{ 2,	0,	sys_ftruncate,	"ftruncate" },		/* 130 */
 	{ 2,	0,	sys_flock,	"flock" },		/* 131 */
-	{ 0,	0,	printargs,	"SYS_132" },		/* 132 */
+	{ 2,	TF,	sys_lstat64,	"lstat64" },		/* 132 */
 	{ 6,	TN,	sys_sendto,	"sendto" },		/* 133 */
 	{ 2,	TN,	sys_shutdown,	"shutdown" },		/* 134 */
 	{ 5,	TN,	sys_socketpair,	"socketpair" },		/* 135 */
 	{ 2,	TF,	sys_mkdir,	"mkdir" },		/* 136 */
 	{ 1,	TF,	sys_rmdir,	"rmdir" },		/* 137 */
 	{ 2,	TF,	sys_utimes,	"utimes" },		/* 138 */
-	{ 0,	TS,	printargs,	"SYS_139" },		/* 139 */
+	{ 2,	TF,	sys_stat64,	"stat64" },		/* 139 */
 	{ 2,   0,	sys_adjtime,	"adjtime" },		/* 140 */
 	{ 3,	TN,	sys_getpeername,"getpeername" },	/* 141 */
 	{ 2,   0,	sys_gethostid,	"gethostid" },		/* 142 */
 	{ 0,	0,	printargs,	"SYS_143" },		/* 143 */
 	{ 2,	0,	sys_getrlimit,	"getrlimit" },		/* 144 */
 	{ 2,	0,	sys_setrlimit,	"setrlimit" },		/* 145 */
-	{ 2,	TS,	sys_killpg,	"killpg" },		/* 146 */
+	{ 2,	TF,	sys_pivotroot,	"pivot_root" },		/* 146 */
+/*	{ 2,	TS,	sys_killpg,	"killpg" },		   146 SunOS killpkg, overridden by Linux pivot_root */
 	{ 5,	0,	printargs,	"prctl" },		/* 147 */
 	{ 5,	0,	printargs,	"pciconfig_read" },	/* 148 */
 	{ 5,	0,	printargs,	"pciconfig_write" },	/* 149 */
@@ -152,7 +153,7 @@
 	{ 4,	TN,	sys_getmsg,	"getmsg" },		/* 151 */
 	{ 4,	TN,	sys_putmsg,	"putmsg" },		/* 152 */
 	{ 3,	0,	sys_poll,	"poll" },		/* 153 */
-	{ 0,	0,	printargs,	"SYS_154" },		/* 154 */
+	{ 4,	0,	printargs,	"getdents64" },		/* 154 */
 	{ 1,	0,	printargs,	"nfssvc" },		/* 155 */
 	{ 4,	0,	printargs,	"getdirentries" },	/* 156 */
 	{ 2,	TF,	sys_statfs,	"statfs" },		/* 157 */
@@ -241,7 +242,7 @@
 	{ 5,	0,	sys_mlock,	"mlock" },		/* 237 */
 	{ 5,	0,	sys_munlock,	"munlock" },		/* 238 */
 	{ 5,	0,	sys_mlockall,	"mlockall" },		/* 239 */
-	{ 5,	0,	sys_munlockall,	"munlockall" },		/* 240 */
+	{ 0,	0,	sys_munlockall,	"munlockall" },		/* 240 */
 	{ 5,	0,	sys_sched_setparam,"sched_setparam"},	/* 241 */
 	{ 5,	0,	sys_sched_getparam,"sched_getparam"},	/* 242 */
 { 5,	0,	sys_sched_setscheduler,"sched_setscheduler"},	/* 243 */
