@@ -25,7 +25,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	$Id: syscallent.h,v 1.9 2001/03/19 11:05:17 wichert Exp $
+ *	$Id: syscallent.h,v 1.10 2001/08/03 12:27:54 wichert Exp $
  */
 
 	{ 0,	0,	sys_setup,		"setup"		}, /* 0 */
@@ -267,7 +267,7 @@
 	{ 3,	0,	printargs,		"mincore"	}, /* 218 */
 	{ 3,	0,	sys_madvise,		"madvise"	}, /* 219 */
 	{ 4,	0,	printargs,		"getdents64"	}, /* 220 */
-	{ 4,	0,	printargs,		"fcntl64"	}, /* 221 */
+	{ 3,	0,	sys_fcntl,		"fcntl64"	}, /* 221 */
 	{ 4,	0,	printargs,		"SYS_222"	}, /* 222 */
 	{ 4,	0,	printargs,		"SYS_223"	}, /* 223 */
 	{ 4,	0,	printargs,		"SYS_224"	}, /* 224 */
