@@ -31,7 +31,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	$Id: process.c,v 1.28 2001/03/28 20:29:17 wichert Exp $
+ *	$Id: process.c,v 1.31 2001/08/03 11:51:28 wichert Exp $
  */
 
 #include "defs.h"
@@ -366,6 +366,26 @@ struct tcb *tcp;
 	return 0;
 }
 
+#if UNIXWARE > 2
+
+int
+sys_rfork(tcp)
+struct tcb *tcp;
+{
+	if (entering(tcp)) {
+		tprintf ("%ld", tcp->u_arg[0]);
+	}
+	else {
+		if (getrval2(tcp)) {
+			tcp->auxstr = "child process";
+			return RVAL_UDECIMAL | RVAL_STR;
+		}
+	}
+	return 0;
+}
+
+#endif
+
 int
 internal_fork(tcp)
 struct tcb *tcp;
@@ -460,7 +480,7 @@ int new;
 		return -1;
 	return 0;
 #elif defined(POWERPC)
-	if (ptrace(PTRACE_POKEUSER, tcp->pid, (CHAR*)(4*PT_R0), new) < 0)
+	if (ptrace(PTRACE_POKEUSER, tcp->pid, (char*)(4*PT_R0), new) < 0)
 		return -1;
 #elif defined(S390)
 	long	pc;
@@ -1185,6 +1205,20 @@ struct tcb *tcp;
 	return 0;
 }
 
+#if UNIXWARE > 2
+
+int sys_rexecve(tcp)
+struct tcb *tcp;
+{
+	if (entering (tcp)) {
+		sys_execve (tcp);
+		tprintf (", %ld", tcp->u_arg[3]);
+	}
+	return 0;
+}
+
+#endif
+
 int
 internal_exec(tcp)
 struct tcb *tcp;
@@ -1989,16 +2023,16 @@ struct xlat struct_user_offsets[] = {
 	{ PT_F27, "f27" }, { PT_F28, "f28" }, { PT_F29, "f29" },
 	{ PT_F30, "f30" }, { PT_F31, "f31" }, { PT_R4, "r4" },
 	{ PT_R5, "r5" }, { PT_R6, "r6" }, { PT_R7, "r7" },
-	{ PT_K_B0, "kb0" },
+	{ PT_B0, "kb0" },
 	{ PT_B1, "b1" }, { PT_B2, "b2" }, { PT_B3, "b3" },
 	{ PT_B4, "b4" }, { PT_B5, "b5" },
-	{ PT_K_AR_PFS, "kar.pfs" },
-	{ PT_AR_LC, "ar.lc" }, { PT_K_AR_UNAT, "kar.unat" },
-	{ PT_K_AR_RNAT, "kar.rnat" }, { PT_K_AR_BSPSTORE, "kar.bspstore" },
-	{ PT_K_PR, "k.pr" },
+	{ PT_AR_PFS, "kar.pfs" },
+	{ PT_AR_LC, "ar.lc" }, { PT_AR_UNAT, "kar.unat" },
+	{ PT_AR_RNAT, "kar.rnat" }, { PT_AR_BSPSTORE, "kar.bspstore" },
+	{ PT_PR, "k.pr" },
 	/* pt_regs */
 	{ PT_CR_IPSR, "cr.ipsr" }, { PT_CR_IIP, "cr.iip" },
-	{ PT_CR_IFS, "cr.ifs" }, { PT_AR_UNAT, "ar.unat" },
+	/*{ PT_CR_IFS, "cr.ifs" },*/ { PT_AR_UNAT, "ar.unat" },
 	{ PT_AR_PFS, "ar.pfs" }, { PT_AR_RSC, "ar.rsc" },
 	{ PT_AR_RNAT, "ar.rnat" }, { PT_AR_BSPSTORE, "ar.bspstore" },
 	{ PT_PR, "pr" }, { PT_B6, "b6" }, { PT_AR_BSP, "ar.bsp" },
