@@ -27,7 +27,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	$Id: strace.c,v 1.21 2001/03/28 14:40:14 wichert Exp $
+ *	$Id: strace.c,v 1.23 2001/08/03 11:43:35 wichert Exp $
  */
 
 #include <sys/types.h>
@@ -51,9 +51,11 @@
 #ifdef SVR4
 #include <sys/stropts.h>
 #ifdef HAVE_MP_PROCFS
+#ifdef HAVE_SYS_UIO_H
 #include <sys/uio.h>
 #endif
 #endif
+#endif
 
 int debug = 0, followfork = 0, followvfork = 0, interactive = 0;
 int rflag = 0, tflag = 0, dtime = 0, cflag = 0;
@@ -312,11 +314,23 @@ char *argv[];
 
 	/* Check if they want to redirect the output. */
 	if (outfname) {
+		long f;
+
 		if ((outf = fopen(outfname, "w")) == NULL) {
 			fprintf(stderr, "%s: can't fopen '%s': %s\n",
 				progname, outfname, strerror(errno));
 			exit(1);
 		}
+
+		if ((f=fcntl(fileno(outf), F_GETFD)) < 0 ) {
+			perror("failed to get flags for outputfile");
+			exit(1);
+		}
+
+		if (fcntl(fileno(outf), F_SETFD, f|FD_CLOEXEC) < 0 ) {
+			perror("failed to set flags for outputfile");
+			exit(1);
+		}
 	}
 
 #ifndef SVR4
