@@ -30,7 +30,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	$Id: syscall.c,v 1.30 2001/03/28 20:29:17 wichert Exp $
+ *	$Id: syscall.c,v 1.32 2001/07/10 13:48:44 hughesj Exp $
  */
 
 #include "defs.h"
@@ -417,6 +417,19 @@ struct tcb *tcp;
 		if (qual_flags[tcp->u_arg[0]] & QUAL_WRITE)
 			dumpstr(tcp, tcp->u_arg[1], tcp->u_arg[2]);
 		break;
+#ifdef SYS_readv
+        case SYS_readv:
+                if (qual_flags[tcp->u_arg[0]] & QUAL_READ)
+                        dumpiov(tcp, tcp->u_arg[2], tcp->u_arg[1]);
+                break;
+#endif
+#ifdef SYS_writev
+        case SYS_writev:
+
+                if (qual_flags[tcp->u_arg[0]] & QUAL_WRITE)
+                        dumpiov(tcp, tcp->u_arg[2], tcp->u_arg[1]);
+                break;
+#endif
 	}
 }
 
@@ -577,6 +590,18 @@ struct tcb *tcp;
 #endif
 #ifdef SYS_vfork
 	case SYS_vfork:
+#endif
+#ifdef SYS_fork1
+	case SYS_fork1:
+#endif
+#ifdef SYS_forkall
+	case SYS_forkall:
+#endif
+#ifdef SYS_rfork1
+	case SYS_rfork1:
+#endif
+#ifdef SYS_rforkall
+	case SYS_rforkall:
 #endif
 		internal_fork(tcp);
 		break;
@@ -590,6 +615,9 @@ struct tcb *tcp;
 #endif
 #ifdef SYS_execve
 	case SYS_execve:
+#endif
+#ifdef SYS_rexecve
+	case SYS_rexecve:
 #endif
 		internal_exec(tcp);
 		break;
@@ -879,6 +907,18 @@ struct tcb *tcp;
 #ifdef SYS_vfork
 			    || scno == SYS_vfork
 #endif /* SYS_vfork */
+#ifdef SYS_fork1
+			    || scno == SYS_fork1
+#endif /* SYS_fork1 */
+#ifdef SYS_forkall
+			    || scno == SYS_forkall
+#endif /* SYS_forkall */
+#ifdef SYS_rfork1
+			    || scno == SYS_rfork1
+#endif /* SYS_fork1 */
+#ifdef SYS_rforkall
+			    || scno == SYS_rforkall
+#endif /* SYS_rforkall */
 			    ) {
 				/* We are returning in the child, fake it. */
 				tcp->status.PR_WHY = PR_SYSENTRY;
