@@ -30,7 +30,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	$Id: util.c,v 1.22 2001/03/27 12:17:17 wichert Exp $
+ *	$Id: util.c,v 1.25 2001/07/10 13:48:44 hughesj Exp $
  */
 
 #include "defs.h"
@@ -38,6 +38,9 @@
 #include <sys/user.h>
 #include <sys/param.h>
 #include <fcntl.h>
+#if HAVE_SYS_UIO_H
+#include <sys/uio.h>
+#endif
 #ifdef SUNOS4
 #include <machine/reg.h>
 #include <a.out.h>
@@ -464,6 +467,38 @@ int len;
 	tprintf("%s", outstr);
 }
 
+#if HAVE_SYS_UIO_H
+void
+dumpiov(tcp, len, addr)
+struct tcb * tcp;
+int len;
+long addr;
+{
+	struct iovec *iov;
+	int i;
+
+	  
+	if ((iov = (struct iovec *) malloc(len * sizeof *iov)) == NULL) {
+		fprintf(stderr, "dump: No memory");
+		return;
+	}
+	if (umoven(tcp, addr,
+		   len * sizeof *iov, (char *) iov) >= 0) {
+                
+		for (i = 0; i < len; i++) {
+                        /* include the buffer number to make it easy to
+                         * match up the trace with the source */
+                        tprintf(" * %lu bytes in buffer %d\n",
+                                (unsigned long)iov[i].iov_len, i);
+                        dumpstr(tcp, (long) iov[i].iov_base,
+                                iov[i].iov_len);
+                }
+	}
+	free((char *) iov);
+        
+}
+#endif
+
 void
 dumpstr(tcp, addr, len)
 struct tcb *tcp;
@@ -632,25 +667,13 @@ char *laddr;
 
 #ifdef USE_PROCFS
 #ifdef HAVE_MP_PROCFS
-	if (pread(tcp->pfd_as, laddr, len, addr) == -1)
-		return -1;
+	int fd = tcp->pfd_as;
 #else
-/*
- * We would like to use pread preferentially for speed
- * but even though SGI has it in their library, it no longer works.
- */
-#ifdef MIPS
-#undef HAVE_PREAD
+	int fd = tcp->pfd;
 #endif
-#ifdef HAVE_PREAD
-	if (pread(tcp->pfd, laddr, len, addr) == -1)
+	lseek(fd, addr, SEEK_SET);
+	if (read(fd, laddr, len) == -1)
 		return -1;
-#else /* !HAVE_PREAD */
-	lseek(tcp->pfd, addr, SEEK_SET);
-	if (read(tcp->pfd, laddr, len) == -1)
-		return -1;
-#endif /* !HAVE_PREAD */
-#endif /* HAVE_MP_PROCFS */
 #endif /* USE_PROCFS */
 
 	return 0;
@@ -668,7 +691,34 @@ int len;
 char *laddr;
 {
 #ifdef USE_PROCFS
-	return umoven(tcp, addr, len, laddr);
+#ifdef HAVE_MP_PROCFS
+	int fd = tcp->pfd_as;
+#else
+	int fd = tcp->pfd;
+#endif
+	/* Some systems (e.g. FreeBSD) can be upset if we read off the
+	   end of valid memory,  avoid this by trying to read up
+	   to page boundaries.  But we don't know what a page is (and
+	   getpagesize(2) (if it exists) doesn't necessarily return
+	   hardware page size).  Assume all pages >= 1024 (a-historical
+	   I know) */
+
+	int page = 1024; 	/* How to find this? */
+	int move = page - (addr & (page - 1));
+	int left = len;
+
+	lseek(fd, addr, SEEK_SET);
+
+	while (left) {
+		if (move > left) move = left;
+		if ((move = read(fd, laddr, move)) == -1)
+			return left != len ? 0 : -1;
+		if (memchr (laddr, 0, move)) break;
+		left -= move;
+		laddr += move;
+		addr += move;
+		move = page;
+	}
 #else /* !USE_PROCFS */
 	int started = 0;
 	int pid = tcp->pid;
@@ -718,8 +768,8 @@ char *laddr;
 
 		addr += sizeof(long), laddr += m, len -= m;
 	}
-	return 0;
 #endif /* !USE_PROCFS */
+	return 0;
 }
 
 #ifdef LINUX
@@ -996,6 +1046,14 @@ struct tcb *tcp;
 		return;
 	}
 	tprintf("[%08lx] ", pc);
+#elif defined(MIPS)
+	long pc;
+
+	if (upeek(tcp->pid, REG_EPC, &pc) < 0) {
+		tprintf ("[????????] ");
+		return;
+	}
+	tprintf("[%08lx] ", pc);
 #endif /* !architecture */
 #endif /* LINUX */
 
